#ifndef TOUCHBTNS_H
#define TOUCHBTNS_H

#include "propeller.h"

extern uint32_t _load_start_TouchBtns_cog[];
extern uint32_t _load_stop_TouchBtns_cog[];

typedef volatile struct TB_Mailbox 
{
    uint32_t pins;      /* Pins to sample as touch buttons */
    uint32_t delay;     /* Delay between button samples */
    uint32_t btnStates; /* The most recent state of the buttons */
} Mailbox_t;

/*
 * This defines the structure which we'll pass to the C cog.
 */
struct par_s 
{
    unsigned int stack[16]; /* Provide a small stack for the COG */
    Mailbox_t mailbox;      /* Provide a mailbox to communicate to the COG */
};


#if defined(__cplusplus)

class TouchBtns
{
    public:
        TouchBtns(uint8_t bpin, uint8_t count, uint32_t rate, uint8_t samples)
        {//
         // Initialize the object variables.  [bpin] is the lowest starting I/O
         // pin for the buttons.  [count] is the number of contigous pins to act
         // as buttons.  [rate] is the number of times per second to update the 
         // the pin states.  [samples] is the number of samples that each bit must
         // remain set for to count.
         //
            m_Par.mailbox.pins = 0;

            /* setup the pin mask based on parameters */
            for(int i=count-1; i>=0; i--)
            {
                m_Par.mailbox.pins |= 1 << bpin+i;            
            }

            if (rate > MAX_RATE)
                rate = MAX_RATE;
            else if (rate < MIN_RATE)
                rate = MIN_RATE;

            m_Par.mailbox.delay = CLKFREQ/rate;
            m_Par.mailbox.btnStates = 0;
            m_Rdy = 1;
            m_Cog = -1;
            m_BPin = bpin;
            m_PCount = count;
        }

        int8_t Start(void)
        {//
         // Start a new COG running the TouchBtns.cogc code
         //
            if (m_Rdy)
            {            
                #if defined(__PROPELLER_XMMC__) || defined(__PROPELLER_XMM__)
                    int size = _load_stop_TouchBtns_cog - _load_start_TouchBtns_cog;
                    unsigned int cogbuffer[size];
                    /* memcpy does bytes, so we copy numbytes * 4 = num ints. */      
                    memcpy(cogbuffer, _load_start_TouchBtns_cog, size<<2);
                #else
                    int *cogbuffer = (int*)_load_start_TouchBtns_cog;
                #endif
                m_Cog = cognew(cogbuffer, &m_Par.mailbox);
            }
            else
                m_Cog = -1;

            return m_Cog;
        }

        int8_t Stop()
        {//
         // Stop the running cog
         //
            if (m_Cog != -1)
            {
                cogstop(m_Cog);
                m_Cog = -1;
            }
            
            return m_Cog;
        }

        int32_t GetButtons(void)
        {//
         // Returns the state of all of the touch button pins as a single
         // binary number that begins at bit 0 reguadless of actual pin 
         // position.
         //
            int32_t accum;
            uint8_t i = m_Samples-1; 
            
            // Set the accumulator to the initial state of the buttons
            accum = m_Par.mailbox.btnStates;
            
            // Sample the buttons [m_Samples] times, and only bits that are
            // set for all samples survive. 
            while(i)
            { 
                accum &= m_Par.mailbox.btnStates;
                i--;
            }
            
            // Clear bits that aren't buttons and right justify results.
            accum &= m_Par.mailbox.pins;            
            accum >>= m_BPin;                        
            
            return accum;
        }

        int8_t GetButton(uint8_t n)
        {//
         // Return the value of an individual touch button.  Buttons are indexed
         // from 0 to [m_PCount] regaurdlles of actual pin position.
         //
            int32_t allBtns = GetButtons();
            int8_t  result  = 0;

            // Limit the request to the number of buttons we actually control.
            if (n > m_PCount)        
                n = m_PCount;

            n = 1 << n;             // Set the bit for the button we want.                    
            result = allBtns & n;   // Test the button bit to see if it is set

            return result;
        }

        private:
            par_s   m_Par;
            uint8_t m_Rdy;
            int8_t  m_Cog;
            uint8_t m_Samples;
            uint8_t m_BPin;
            uint8_t m_PCount;

            enum
            {
                MAX_RATE = 300,
                MIN_RATE = 90
            }; 
            
};

#endif  // __cplusplus


#endif  // TOUCHBTNS_H